﻿const
	script_name='amazon';
	language='Multiple';
	version=6;
	author='Fade';
	MinProgramBuild=10000;

var
  ASIN: string;
  Title: string;
  Year: string;
  Studio: string;
  MediaURL: string;
  ImageURL: string;
  Duration: Integer;
  Resolution: string;
  MPAARating: string;
  Description: string;
  AspectRatio: string;
  MediaFormat: string;
  ISBNs: TStringList;
  Genres: TStringList;
  Actors: TStringList;
  Writers: TStringList;
  AmazonDomain: string;
  AmazonDomainIDX: Integer;
  Languages: TStringList;
  Subtitles: TStringList;
  Directors: TStringList;

const
  AMAZON_DEFAULT_DOMAIN = 'com';
  AMAZON_ITEM_INFO_BY_ASIN_URL = 'https://www.amazon.%s/dp/%s/';
  AMAZON_SEARCH_ASIN_BY_BARCODE_URL = 'https://www.amazon.%s/s?k=%s';
  UPC_SEARCH_ASIN_BY_BARCODE_URL = 'https://www.upcitemdb.com/upc/%s';

const
  MEDIA_TYPE_MOVIES = 0;
  MEDIA_TYPE_MUSIC = 1;
  MEDIA_TYPE_BOOKS = 2;
  MEDIA_TYPE_GAMES = 3;

const
  AMAZON_DOMAIN_COM_IDX = 0;
  AMAZON_DOMAIN_IT_IDX = 1;
  AMAZON_DOMAIN_DE_IDX = 2;
  AMAZON_DOMAIN_CA_IDX = 3;
  AMAZON_DOMAIN_FR_IDX = 4;

const
  // Arrays with translates of amazon templates for different languages
  // If translated text in an array is empty, english text is used instead (first element of an array)
  AMAZON_TRANSLATE_PUBLISHER = ['Publisher', '', 'Verlag', '', ''];
  AMAZON_TRANSLATE_DIMENSIONS = ['Product Dimensions', '', 'Verpackungsabmessungen', '', ''];
  AMAZON_TRANSLATE_PAGES_PRINT = ['Print Length', '', 'Gebundene Ausgabe', '', ''];
  AMAZON_TRANSLATE_PAGES_PAPERBACK = ['Paperback', '', '', '', ''];
  AMAZON_TRANSLATE_PAGES_HARDCOVER = ['Hardcover', '', '', '', ''];
  AMAZON_TRANSLATE_DETAILS = ['<h2>Product details</h2>', '', '<h2>Produktinformation</h2>', '', ''];

  // list with all amazon translates. Add New list here on adding new translate array
  AMAZON_TRANSLATES_COUNT = 6;
  AMAZON_TRANSLATES = [AMAZON_TRANSLATE_PUBLISHER, AMAZON_TRANSLATE_DIMENSIONS,
    AMAZON_TRANSLATE_PAGES_PRINT, AMAZON_TRANSLATE_PAGES_PAPERBACK,
    AMAZON_TRANSLATE_PAGES_HARDCOVER, AMAZON_TRANSLATE_DETAILS];

var
  DataContent: string;
  ProductDetailsContent: string;

function GetTranslatedArrayIDX(AEnglishText: string): Integer;
var
  I: Integer;
  StrArray: array of string;
begin
  Result := -1;
  AEnglishText := WideLowerCase(AEnglishText);
  for I := 0 to Pred(AMAZON_TRANSLATES_COUNT) do
  begin
    StrArray := AMAZON_TRANSLATES[I];
    if (WideLowerCase(StrArray[AMAZON_DOMAIN_COM_IDX]) = AEnglishText) then
    begin
      Result := I;
      Break;
    end;
  end;
end;

function GetTranslateIndexByDomain(ADomain: string): Integer;
begin
  Result := AMAZON_DOMAIN_COM_IDX;
  ADomain := WideLowerCase(ADomain);
  if (ADomain = 'it') then
    Result := AMAZON_DOMAIN_IT_IDX
  else if (ADomain = 'de') then
    Result := AMAZON_DOMAIN_DE_IDX
  else if (ADomain = 'ca') then
    Result := AMAZON_DOMAIN_CA_IDX
  else if (ADomain = 'fr') then
    Result := AMAZON_DOMAIN_FR_IDX;
end;

function GetTranslatedText(AEnglishText: string): string;
var
  I: Integer;
  ArrIdx: Integer;
  StrArray: array of string;
begin
  Result := AEnglishText;
  ArrIdx := GetTranslatedArrayIDX(AEnglishText);
  if (ArrIdx >= 0) then
  begin
    StrArray := AMAZON_TRANSLATES[ArrIdx];
    if (StrArray[AmazonDomainIDX] <> '') then
      Result := StrArray[AmazonDomainIDX];
  end;
end;

function ProcessSearchBarcode(ABarcode: string): string;
begin
  // extract barcode for searching from any barcode
  Result := ABarcode;
  Result := ReplaceStr(Result, '-', '', -1);
  Result := ReplaceStr(Result, ' ', '', -1);
end;

function ProcessDoubleStr(ADoubleStr: string): string;
begin
  Result := ReplaceStr(ADoubleStr, '.', DecimalSeparator, -1);
  Result := ReplaceStr(Result, ',', DecimalSeparator, -1);
end;

function RemoveSubStr(AString: string; ASubString: string): string;
begin
  Result := Trim(ReplaceStr(AString, ASubString, '', -1));
end;

function ProcessHTML(AString: string): string;
begin
  Result := ConvertUTF8toASCII(AString);
  Result := DecodeHTML(Result);
  Result := ReplaceStr(Result, '<br>', #13#10, -1);
  Result := ReplaceStr(Result, '<BR>', #13#10, -1);
  Result := RemoveTags(Result);
  Result := Trim(Result);
end;

//==================================================================================================
//  Help functions
//==================================================================================================

function ParseTagFromEnd(AText: string; ATag: string): string;
begin
  // search given ATag from the end of text and return text before this tag
  AText := ReverseString(AText);
  ATag := ReverseString(ATag);
  Result := ReverseString(ParseTagFromStart(AText, ATag));
end;

function ParseTagFromEndIfExists(AText: string; ATag: string): string;
begin
  // search given ATag from the end of text and return text before this tag
  // If tag wasn't found - return given AText
  Result := ParseTagFromEnd(AText, ATag);
  if (Result = '') then
    Result := AText;
end;

function ParseAmazonYear(ADateStr: string): string;
var
  YearStr: string;
  YearInt: Integer;
begin
  // try to parse year from Amazon date
  Result := '';
  YearStr := Trim(ParseTagFromEnd(ADateStr, ','));
  if (TryStrToInt(YearStr, YearInt)) then
    Result := YearStr;
end;

procedure AddItemToStringList(AItem: string; AList: TStringList);
begin
  // add string to string list (with checking of string existing)
  AItem := ProcessHTML(AItem);
  if ((AItem <> '') and
      (AList.IndexOf(AItem) < 0)) then
  begin
    AList.Add(AItem);
  end;
end;

procedure ClearFields();
begin
  // clear all variables
  ISBNs.Clear();
  Actors.Clear();
  Genres.Clear();
  Writers.Clear();
  Languages.Clear();
  Subtitles.Clear();
  Directors.Clear();
  Title := '';
  Year := '';
  Studio := '';
  ImageURL := '';
  Duration := 0;
  Description := '';
  Resolution := '';
  MPAARating := '';
  AspectRatio := '';
  MediaFormat := '';
end;

// method converts inches to sm
function InchesToSm(AInches: Double): Double;
const
  INCH_RATIO = 2.54;
begin
  Result := AInches * INCH_RATIO;
end;

// method converts ounces to grams
function OuncesToGram(AOunces: Double): Double;
const
  OUNCES_RATIO = 28.3495;
begin
  Result := AOunces * OUNCES_RATIO;
end;


//==================================================================================================
//  Generate URL functions
//==================================================================================================

// generate amazon info Page URL by ASIN
function GetItemInfoURLByASIN(AASIN: string; AAmazonDomain: string = ''): string;
begin
  Result := '';
  if (AASIN <> '') then
    Result := Format(AMAZON_ITEM_INFO_BY_ASIN_URL, [AAmazonDomain, AASIN]);
end;

// generate search ASIN URL by barcode at UPCDB server
function GetSearchUrlByBarcodeUPCDB(ABarcode: string): string;
begin
  Result := '';
  ABarcode := ProcessSearchBarcode(ABarcode);
  if (ABarcode <> '') then
    Result := Format(UPC_SEARCH_ASIN_BY_BARCODE_URL, [ABarcode]);
end;

// generate search ASIN URL by barcode at Amazon server
function GetSearchURLByBarcodeAmazon(ABarcode: string; AAmazonDomain: string): string;
begin
  Result := '';
  ABarcode := ProcessSearchBarcode(ABarcode);
  if (AAmazonDomain = '') then
    AAmazonDomain := AMAZON_DEFAULT_DOMAIN;
  if (ABarcode <> '') then
    Result := Format(AMAZON_SEARCH_ASIN_BY_BARCODE_URL, [AAmazonDomain, ABarcode]);
end;

function GetSearchURLByText(AText: string; AAmazonDomain: string): string;
begin
  Result := '';
  if (AAmazonDomain = '') then
    AAmazonDomain := AMAZON_DEFAULT_DOMAIN;
  if (AText <> '') then
  begin
    AText := ReplaceStr(AText, ' ', '+', -1);
    AText := EncodeURL(AText, 'UTF-8');
    Result := Format(AMAZON_SEARCH_ASIN_BY_BARCODE_URL, [AAmazonDomain, AText]);
  end;
end;

function GetAmazonResponseGroupByMediaMode(AMode: Integer): string;
begin
  Result := '';
  case AMode of
    MEDIA_TYPE_MOVIES: Result := 'movies-tv';
    MEDIA_TYPE_MUSIC: Result := 'music';
    MEDIA_TYPE_BOOKS: Result := 'stripbooks';
    MEDIA_TYPE_GAMES: Result := 'videogames';
  end;
  {if (Result <> '') then
    Result := Result + '-intl-ship'}
end;

//==================================================================================================
//  Internal parsing functions
//==================================================================================================

procedure FillStringListFromStr(AList: TStringList; AString: string);
var
  I: Integer;
  Item: string;
  SourceList: TStringList;
begin
  if (Assigned(AList)) then
  begin
    AList.Clear();
    SourceList := StringToStringList(AString, ',');
    try
      for I := 0 to SourceList.Count - 1 do
      begin
        Item := SourceList[I];
        if (Pos('<a', Item) > 0) then
        begin
          Item := ParseTag(Item, '<a', '</a>');
          Item := ParseTagFromEnd(Item, '>');
        end;
        AddItemToStringList(Item, AList);
      end;
    finally
      SourceList.Free();
    end;
  end;
end;

function ParseAmazonProductDetailItem(AItemName: string): string;
var
  SearchTagStart: string;
begin
  AItemName := GetTranslatedText(AItemName);
  SearchTagStart := Format('%s:', [AItemName]);
  Result := Trim(ParseTag(ProductDetailsContent, SearchTagStart, '</li>'));
  if (Pos('</b>', Result) = 1) then
  begin
    Delete(Result, 1, Length('</b>'));
    Result := Trim(Result)
  end;
  Result := Trim(RemoveStartEndBrackets(Result, '"', '"'));
end;

function ParseAmazonProductDetailItemBook(AItemName: string): string;
begin
  Result := ParseTag(ProductDetailsContent, AItemName, '</td>');
  Result := ParseTagFromEnd(Result, '>');
  Result := Trim(RemoveStartEndBrackets(Result, '"', '"'));
end;

function ParseTitle(): string;
var
  Content: string;
begin
  Content := ParseTag(DataContent, '<h1 id="title', '</h1>');
  Content := ParseTag(Content, 'productTitle"', '</span>');
  Result := ProcessHTML(ParseTagFromEnd(Content, '>'));
  // remove "(### edition)"
  if (Pos(ReverseString('Edition)'), ReverseString(Result)) = 1) then
  begin
    Result := ReverseString(Result);
    try
      if (Pos('(', Result) > 0) then
        Delete(Result, 1, Pos('(', Result));
    finally
      Result := ReverseString(Trim(Result));
    end;
  end;
end;

function ParseAuthor(): string;
var
  Content: string;
begin
  Content := ParseTag(DataContent, '<div id="bylineInfo', '</div>');
  Result := ParseTag(Content, '<a', '</a>');
  Result := Trim(ParseTagFromEnd(Result, '>'));
  Result := ParseTagFromEndIfExists(Result, #$A);
  if (Result = '') then
    Result := ParseTag(Content, '<span class="a-size-medium">', '<span');
  Result := ProcessHTML(Result);
end;

function ParseFormat(): string;
var
  Content: string;
begin
  Content := ParseTagTakingIntoAccountNestingLevel(DataContent,
    '<div id="bylineInfo_feature_div"', '<div', '</div>');
  Content := ParseTag(Content, 'Format: ', '');
  Result := ProcessHTML(ParseTag(Content, '<span>', '</span>'));
end;

function ParseImageURL(): string;
var
  Content: string;
begin
  Content := ParseTag(DataContent, '<div id="digitalMusicProductImage_feature_div"', '</div>');
  Result := ParseTag(Content, '<img', '">');
  Result := Trim(ParseTagFromEnd(Result, '"'));
  if (Result = '') then
  begin
    Content := ParseTag(DataContent, 'data-a-dynamic-image="{', '}');
    Result := ParseTag(Content, ';', ';');
    Result := ReverseString(Result);
    try
      // Fast script doesn't understand quotable symbols in the unit. Replace them by string reversing
      if (Pos('touq&', Result) = 1) then
        Delete(Result, 1, Length('touq&'));
    finally
      Result := ReverseString(Result);
    end;
  end;
end;

function ParseAmazonDuration(ADurationStr: string): Integer;
begin
  Result := 0;
  if (Pos(' ', ADurationStr) > 0) then
    ADurationStr := Copy(ADurationStr, 1, Pos(' ', ADurationStr) - 1);
  if (not TryStrToInt(ADurationStr, Result)) then
    Result := 0;
end;

function ParseMPAA(): string;
var
  RatingStr: string;
  RatingDesc: string;
begin
  // parse rating itself
  RatingStr := ParseAmazonProductDetailItem('Rated');
  Result := Trim(ParseTag(RatingStr, '<span class="a-size-small', '</span>'));
  Result := Trim(ParseTagFromEnd(Result, '>'));
  // parse rating description
  if (Result <> '') then
  begin
    RatingDesc := Trim(ParseTagFromEnd(RatingStr, #$A));
    if (RatingDesc <> '') then
      Result := Format('%s (%s)', [Result, RatingDesc]);
  end;
end;

function ParseMediaFormat(): string;
var
  I: Integer;
  FormatsStr: string;
  FormatsList: TStringList;
const
  GLOBAL_FORMAT_STR = 'Multiple Formats';
begin
  Result := '';
  FormatsStr := ParseAmazonProductDetailItem('Format');
  FormatsList := TStringList.Create();
  try
    FillStringListFromStr(FormatsList, FormatsStr);
    for I := 0 to FormatsList.Count - 1 do
    begin
      if (LowerCase(FormatsList[I]) = LowerCase(GLOBAL_FORMAT_STR)) then
        Continue;
      Result := FormatsList[I];
      Break;
    end;
  finally
    FormatsList.Free();
  end;
end;

procedure ParseGenres();
var
  GenresStr: string;
  GenreItemStr: string;
const
  GENERAL_GENRE = 'General';
begin
  GenresStr := ParseTag(ProductDetailsContent, '<ul class="zg_hrsr', '</ul>');
  if (GenresStr <> '') then
  begin
    repeat
      GenreItemStr := ParseTagWithCutting(GenresStr, '<li class="zg_hrsr_item', '</li>');
      if (GenreItemStr <> '') then
      begin
        GenreItemStr := ParseTag(GenreItemStr, '<b>', '</b>');
        GenreItemStr := ParseTag(GenreItemStr, '<a ', '</a>');
        GenreItemStr := Trim(ParseTagFromEnd(GenreItemStr, '>'));
        if (Pos('(', GenreItemStr) > 0) then
          GenreItemStr := Copy(GenreItemStr, 1, Pos(' ', GenreItemStr) - 1);
        GenreItemStr := RemoveSubStr(GenreItemStr, 'DVDs');
        if (LowerCase(GenreItemStr) <> LowerCase(GENERAL_GENRE)) then
          AddItemToStringList(GenreItemStr, Genres);
      end;
    until (GenreItemStr = '');
  end;
end;

function ProcessTrackTitle(ATrackTitle: string): string;
begin
  Result := ATrackTitle;
  if (Pos('[Explicit]', Result) > 0) then
    Result := Copy(Result, 1, Pos('[Explicit]', Result) - 1);
end;

procedure ParsePlayableTracks(ATracksContent: string);
var
  TrackRow: string;
begin
  ATracksContent := ParseTagFromStart(ATracksContent, '</tbody>');
  if (ATracksContent <> '') then
  begin
    repeat
      TrackRow := ParseTagWithCutting(ATracksContent, '<tr id="dmusic_tracklist_player_row', '</tr>');
      if (TrackRow <> '') then
      begin
        TrackRow := ParseTag(TrackRow, 'TrackTitleColumn', '');
        TrackRow := ParseTag(TrackRow, '<a ', '</a>');
        TrackRow := ProcessHTML(ParseTagFromEnd(TrackRow, '>'));
        SetTrack(1, ProcessTrackTitle(TrackRow));
      end;
    until (TrackRow = '');
  end;
end;

procedure ParseUnplayableTracks(ATracksContent: string);
var
  TrackRow: string;
  DiscNumber: Integer;
  DiscNumberStr: string;
begin
  ATracksContent := ParseTag(ATracksContent, '<h1', '');
  ATracksContent := ParseTagFromStart(ATracksContent, '<script>');
  if (ATracksContent <> '') then
  begin
    DiscNumber := 1;
    repeat
      TrackRow := ParseTagWithCutting(ATracksContent, '<div class="a-row', '</div>');
      if (TrackRow <> '') then
      begin
        if (Pos('Disc:', TrackRow) > 0) then
        begin
          // this is disc number - parse it
          DiscNumberStr := ParseTag(TrackRow, 'Disc:', #$A);
          if (not TryStrToInt(DiscNumberStr, DiscNumber)) then
            DiscNumber := 1;
        end
        else
        begin
          // this is track info
          TrackRow := ParseTag(TrackRow, '<span', '<br>');
          TrackRow := ProcessHTML(ParseTagFromEnd(TrackRow, '>'));
          // remove track index
          if (Pos('.', TrackRow) > 0) then
          begin
            Delete(TrackRow, 1, Pos('.', TrackRow));
            TrackRow := Trim(TrackRow);
          end;
          SetTrack(DiscNumber, ProcessTrackTitle(TrackRow));
        end;
      end;
    until (TrackRow = '');
  end;
end;

procedure ParseTracksList();
var
  IsPlayableTracks: Boolean;
  TracksContentStr: string;
begin
  TracksContentStr := ParseTag(DataContent, '<div id="dmusicTracklist_feature_div', '');
  if (TracksContentStr <> '') then
  begin
    IsPlayableTracks := (Pos('id="dmusic_tracklist_header_box', TracksContentStr) > 0);
    if (IsPlayableTracks) then
      ParsePlayableTracks(TracksContentStr)
    else
      ParseUnplayableTracks(TracksContentStr);
  end;
end;

procedure ParseProductDetailsMovies();
var
  NewMediaFormat: string;
begin
  MPAARating := ParseMPAA();
  if (MediaFormat = '') then
    MediaFormat := ParseMediaFormat();
  Studio := ParseAmazonProductDetailItem('Studio');
  AspectRatio := ParseAmazonProductDetailItem('Aspect Ratio');
  Duration := ParseAmazonDuration(ParseAmazonProductDetailItem('Run Time'));
  Year := ParseAmazonYear(ParseAmazonProductDetailItem('DVD Release Date'));
  FillStringListFromStr(Actors, ParseAmazonProductDetailItem('Actors'));
  FillStringListFromStr(Writers, ParseAmazonProductDetailItem('Writers'));
  FillStringListFromStr(Directors, ParseAmazonProductDetailItem('Directors'));
  FillStringListFromStr(Languages, ParseAmazonProductDetailItem('Language'));
  FillStringListFromStr(Subtitles, ParseAmazonProductDetailItem('Subtitles'));
end;

procedure ParseBookDimensions();
var
  WidthStr: string;
  HeightStr: string;
  LengthStr: string;
  WeightStr: string;
  WidthDouble: Double;
  HeightDouble: Double;
  LengthDouble: Double;
  WeightDouble: Double;
  TempInt: Integer;
  DimensionsStr: string;
begin
  DimensionsStr := ParseAmazonProductDetailItem('Product Dimensions');
  WeightStr := ParseAmazonProductDetailItem('Shipping Weight');
  WeightStr := ProcessDoubleStr(ParseTagFromStart(WeightStr, ' '));
  if (DimensionsStr <> '') then
  begin
    WidthStr := ProcessDoubleStr(Trim(ParseTagFromStartWithCutting(DimensionsStr, 'x')));
    HeightStr := ProcessDoubleStr(Trim(ParseTagFromStartWithCutting(DimensionsStr, 'x')));
    DimensionsStr := ProcessDoubleStr(Trim(DimensionsStr));
    LengthStr := Trim(ParseTagFromStartWithCutting(DimensionsStr, ' '));
    if ((TryStrToFloat(WidthStr, WidthDouble)) and
        (TryStrToFloat(HeightStr, HeightDouble)) and
        (TryStrToFloat(LengthStr, LengthDouble))) then
    begin
      Resolution := FormatFloat('0.0', InchesToSm(LengthDouble));
      Resolution := Resolution + 'x' + FormatFloat('0.0', InchesToSm(WidthDouble));
      Resolution := Resolution + 'x' + FormatFloat('0.0', InchesToSm(HeightDouble));
      if (TryStrToFloat(WeightStr, WeightDouble)) then
        Resolution := Resolution + '~' + IntToStr(Trunc(OuncesToGram(WeightDouble)));
    end;
  end;
end;

procedure ParseBookPublicationInfo();
var
  PublishInfo: string;
  PublishDate: string;
begin
  PublishInfo := ParseAmazonProductDetailItem('Publisher');
  if (PublishInfo <> '') then
  begin
    PublishDate := '';
    if (Pos('(', PublishInfo) > 0) then
    begin
      PublishDate := Copy(PublishInfo, Pos('(', PublishInfo) + 1, Length(PublishInfo));
      PublishDate := Trim(RemoveStartEndBrackets(PublishDate, '(', ')'));
      PublishInfo := Copy(PublishInfo, 1, Pos('(', PublishInfo) - 1);
    end;
    if (Pos(';', PublishInfo) > 0) then
      PublishInfo := Copy(PublishInfo, 1, Pos(';', PublishInfo) - 1);
    Studio := ProcessHTML(PublishInfo);
    if ((PublishDate <> '') and (Year = '')) then
      Year := ParseAmazonYear(PublishDate);
  end;
end;

function ParseAlbumYear(): string;
var
  MediaFormatWithYear: string;
begin
  Result := ParseAmazonYear(ParseAmazonProductDetailItem('Original Release Date'));
  if ((Result = '') and
      (MediaFormat <> '')) then
  begin
    MediaFormatWithYear := ParseTag(ProductDetailsContent, '<li><b>' + MediaFormat, '</li>');
    MediaFormatWithYear := Trim(ParseTagFromEnd(MediaFormatWithYear, '</b>'));
    MediaFormatWithYear := Trim(RemoveStartEndBrackets(MediaFormatWithYear, '(', ')'));
    Result := ParseAmazonYear(MediaFormatWithYear);
  end;
end;

function ParseBookPagesCount(): Integer;
begin
  Result := ParseAmazonDuration(ParseAmazonProductDetailItem('Print Length'));
  if (Result = 0) then
    Result := ParseAmazonDuration(ParseAmazonProductDetailItem('Paperback'));
  if (Result = 0) then
    Result := ParseAmazonDuration(ParseAmazonProductDetailItem('Hardcover'));
end;

procedure ParseBooksISBNs();
begin
  AddItemToStringList(ParseAmazonProductDetailItem('ISBN-10'), ISBNs);
  AddItemToStringList(ParseAmazonProductDetailItem('ISBN-13'), ISBNs);
end;

function ParseDescription(): string;
begin
  // var 1
  Result := ParseTag(DataContent, '<script id="bookDesc_override_CSS', '');
  Result := ParseTag(Result, '<noscript>', '</noscript>');
  Result := ParseTag(Result, '<div>', '</div>');
  // var 2
  if (Result = '') then
  begin
    Result := ParseTag(DataContent, '<div id="productDescription"', '</div>');
    Result := ParseTag(Result, '<h3>Product Description</h3>', '<p></p>');
    // skip title
    ParseTagWithCutting(Result, '<p>', '</p>');
    Result := ParseTag(Result, '<p>', '</p>');
  end;
  // var 3
  if (Result = '') then
  begin
     Result := ParseTagTakingIntoAccountNestingLevel(DataContent,
      '<div id="productDescription"', '<div', '</div>');
     Result := Trim(ParseTag(Result, '<p>', '</p>'));
     if (Pos('Product Description', Result) = 1) then
      Delete(Result, 1, Length('Product Description'));
  end;
  Result := ProcessHTML(Result);
end;

function ParseGamePlatform(): string;
begin
  Result := ParseTag(DataContent, '<div id="platformInformation_feature_div"', '</div>');
  Result := ParseTag(Result, '</span>', '<span');
  if (Pos(',', Result) > 0) then
    Result := Copy(Result, 1, Pos(',', Result) - 1);
  Result := ProcessHTML(Result);
  if (Pos('windows', LowerCase(Result)) > 0) then
    Result := 'PC';
end;

function ParseGameESRBRating(): string;
begin
  Result := ParseTag(DataContent, '<div id="vgRating_feature_div', '</div>');
  Result := ParseTag(Result, '<a href', '</a>');
  if (Pos('<i', Result) > 0) then
    Delete(Result, Pos('<i', Result), Length(Result));
  Result := ParseTagFromEnd(Result, '>');
  Result := ProcessHTML(Result);
end;

procedure ParseProductDetailsMusic();
begin
  ParseTracksList();
  Year := ParseAlbumYear();
  Studio := ParseAmazonProductDetailItem('Label');
  Duration := 60 * ParseAmazonDuration(ParseAmazonProductDetailItem('Run Time'));
end;

procedure ParseProductDetailsBooks();
begin
  // pages count
  Duration := ParseBookPagesCount();
  Year := ParseAmazonYear(ParseAmazonProductDetailItem('Publication Date'));
  FillStringListFromStr(Languages, ParseAmazonProductDetailItem('Language'));
  ParseBookPublicationInfo();
  ParseBooksISBNs();
  ParseBookDimensions();
end;

procedure ParseProductDetailsGames();
begin
  Studio := ParseGamePlatform();
  MPAARating := ParseGameESRBRating();
  Year := ParseAmazonYear(ParseAmazonProductDetailItemBook('Release Date'));
end;

function ParseDetailsBlock(): string;
begin
  Result := ParseTag(DataContent, GetTranslatedText('<h2>Product details</h2>'), '</tr>');
  if (Result = '') then
  begin
    Result := ParseTagTakingIntoAccountNestingLevel(DataContent,
      '<div id="prodDetails"', '<div', '</div>');
    Result := ParseTag(Result, ' <table', '</table>');
  end;
end;

procedure ParseProductDetails(AMediaType: Integer);
begin
  ProductDetailsContent := ParseDetailsBlock();
  ParseGenres();
  case AMediaType of
    MEDIA_TYPE_MOVIES: ParseProductDetailsMovies();
    MEDIA_TYPE_MUSIC: ParseProductDetailsMusic();
    MEDIA_TYPE_BOOKS: ParseProductDetailsBooks();
    MEDIA_TYPE_GAMES: ParseProductDetailsGames();
  end;
end;

//==================================================================================================
//  Main parsing functions
//==================================================================================================

// parse ASIN from Amazon search page
function ParseASINFromAmazonContent(AContent: string): string;
begin
  Result := ParseTag(AContent, 'data-asin="', '"');
end;

function ParseImageURLFromAmazonContent(AContent: string): string;
begin
  AContent := ParseTag(AContent, 'data-asin="', '');
  Result := ParseTag(AContent, 'img src="', '"');
end;

// parse ASIN from UPCDB search page
function ParseASINFromUPCDBContent(AContent: string): string;
var
  ASINStr: string;
begin
  Result := '';
  if (AContent <> '') then
  begin
    ASINStr := ParseTag(AContent, '<td>Amazon ASIN', '</tr>');
    Result := Trim(ParseTag(ASINStr, '<td>', '</td>'));
  end;
end;

// parse amazon media information from Amazon Item Page content
function ParseItemInfo(AContent: string; AMediaType: Integer;
  AAmazonDomain: string = 'com'): Boolean;
var
  Author: string;
begin
  Result := (AContent <> '');
  if (Result) then
  begin
    DataContent := AContent;
    AmazonDomain := AAmazonDomain;
    AmazonDomainIDX := GetTranslateIndexByDomain(AAmazonDomain);
    // item data parsing
    ClearFields();
    Title := ParseTitle();
    ImageURL := ParseImageURL();
    Author := ParseAuthor();
    if (Author <> '') then
      Actors.Add(Author);
    MediaFormat := ParseFormat();
    Description := ParseDescription();
    ASIN := ParseASINFromAmazonContent(DataContent);
    MediaURL := GetItemInfoURLByASIN(ASIN, AAmazonDomain);
    ParseProductDetails(AMediaType);
  end;
end;

begin
  ISBNs := TStringList.Create();
  Actors := TStringList.Create();
  Genres := TStringList.Create();
  Writers := TStringList.Create();
  Languages := TStringList.Create();
  Subtitles := TStringList.Create();
  Directors := TStringList.Create();
end.
