﻿program modern_program_global;

uses 'modern_program_variables.pas';

procedure GenerateSQLForMoviesPages(ASQL: TStringList; AProgramType: Integer;
  AMode: Integer; AMaxRecordsCount: Integer = -1);

var
  I: Integer;
  current_site: string;
  ShowLastUpdated: integer;
  IndexByFieldsName: string;
  OrderByFieldsName: string;
begin
  SetRegistrySubKey(REGISTRY_SUB_KEYS[AProgramType]);
  try
    ShowLastUpdated := RegReadInteger('ShowLastUpdated', 0);
    //Get movies
    IndexByFieldsName := 'ind_movies_rating_date_add_desc';
    OrderByFieldsName := 'rating desc, date_add desc';
    if (AMode <> 2) then
    begin
      OrderByFieldsName := 'date_add desc';
      IndexByFieldsName := 'ind_movies_date_add_desc';
      if ((AMode <> 1) and
          (ShowLastUpdated = 0)) then
      begin
        OrderByFieldsName := 'date_update desc';
        IndexByFieldsName := 'ind_movies_date_update_desc';
      end;
    end;
    with ASQL do
    begin
      Clear;
      Add('SELECT *,');
      Add('substr(ImageInfo, 1, 1) as mode,');
      Add('substr(ImageInfo, 3, 5) as width,');
      Add('substr(ImageInfo, 9, 5) as height,');
      Add('substr(ImageInfo, 15, length(ImageInfo) - 2) as path');
      Add('FROM (');
      Add('select code, series_nom, title, title_sort, year, imdb_rating, date_add, item_type, date_update, rating');
      Add(', (SELECT mode || '';'' || substr(''00000'' || width, -5, 5) || '';'' || substr(''00000'' || height, -5, 5) || '';'' || path FROM images iim');
      Add('WHERE iim.movie = movies.code');
      Add('AND iim.mode = 1');
      Add('AND IFNULL(iim.sort_order, 0) = (select IFNULL(Min(Sort_order), 0) from images where movie = iim.movie and mode = 1)) as ImageInfo');
      Add('FROM movies');
      if (IndexByFieldsName <> '') then
        Add('INDEXED BY ' + IndexByFieldsName);
      Add('WHERE in_collection=1');
      Add('AND item_type = ' + IntToStr(AProgramType));
      case AMode of
        3: Add('AND seen<>1');
      end;
      if (RegReadInteger('ShowDummyCoversNew', 1) = 0) then
        Add('AND ImageInfo is not NULL');

      if (OrderByFieldsName <> '') then
        Add('ORDER BY ' + OrderByFieldsName);
      if (AMaxRecordsCount > 0) then
        Add('LIMIT ' + IntToStr(AMaxRecordsCount));
      Add(') as MoviesSubQuery');
      if (OrderByFieldsName = '') then
        OrderByFieldsName := 'Code'
      else
        OrderByFieldsName := OrderByFieldsName + ', Code';
      if (OrderByFieldsName <> '') then
        Add('ORDER BY ' + OrderByFieldsName);
    end;
  finally
    ResetRegistrySubKey();
  end;
end;

procedure GenerateSQLForPersonPages(ASQL: TStringList; AMode: Integer;
  AMaxRecordsCount: Integer = -1);
var
  I: Integer;
  AdditionalSQL: string;
  OrderByFieldsName: string;
  DateDay: Integer;
  DateYear: Integer;
  DateMonth: Integer;
  ProgramTypesSQL: string;
  ShowLastUpdated: integer;
begin
  OrderByFieldsName := '';
  SetRegistrySubKey(REGISTRY_SUB_KEYS[MOVIENIZER_PROGRAM]);
  try
    ShowLastUpdated := RegReadInteger('ShowLastUpdated', 0);
    //Get persons
    with ASQL do
    begin
      Clear;
      Add('SELECT *,');
      Add('substr(ImageInfo, 1, 1) as mode,');
      Add('substr(ImageInfo, 3, 5) as width,');
      Add('substr(ImageInfo, 9, 5) as height,');
      Add('substr(ImageInfo, 15, length(ImageInfo) - 2) as path');
      Add('FROM (');
      Add('select code, name, date_add, date_update, rating, item_type, birth_date, death_date, ');
      Add(' (SELECT mode || '';'' || substr(''00000'' || width, -5, 5) || '';'' || substr(''00000'' || height, -5, 5) || '';'' || path FROM images iim');
      Add('WHERE iim.person = persons.code');
      Add('AND iim.mode = 6');
      Add('AND IFNULL(iim.sort_order, 0) = (select IFNULL(Min(Sort_order), 0) from images where person = iim.person and mode = 6)) as ImageInfo');
      Add('from persons');
      AdditionalSQL := '';
      case AMode of
        2:
        begin
          OrderByFieldsName := 'date_update desc';
          Add('INDEXED BY ind_persons_date_update_desc');
        end;
        3:
        begin
          OrderByFieldsName := 'rating desc, date_add desc';
          Add('INDEXED BY ind_persons_rating_date_add_desc');
        end;
        4: AdditionalSQL := 'AND birth_date like ''%' + Format('%s', [GetTodaysDate]) + '%''';
        else
        begin
          if ShowLastUpdated = 0 then
          begin
            OrderByFieldsName := 'date_add desc';
            Add('INDEXED BY ind_persons_date_add_desc');
          end
          else
          begin
            OrderByFieldsName := 'date_update desc';
            Add('INDEXED BY ind_persons_date_update_desc');
          end;
        end;
      end;
      Add('WHERE in_collection=1');
      Add(AdditionalSQL);
      // generate SQL for select persons with or without covers
      // I is Program Type
      ProgramTypesSQL := '';
      for I := LOW_PROGRAM_TYPE to HIGH_PROGRAM_TYPE do
      begin
        if (IsProgramUsed(I)) then
        begin
          SetRegistrySubKey(REGISTRY_SUB_KEYS[I]);
          try
            if (ProgramTypesSQL <> '') then
              ProgramTypesSQL := ProgramTypesSQL + ' OR ';
            ProgramTypesSQL := ProgramTypesSQL + ' (item_type = ' + IntToStr(I) + ')';
          finally
            ResetRegistrySubKey();
          end;
        end;
      end;
      if (ProgramTypesSQL <> '') then
        Add('AND (' + ProgramTypesSQL + ')');

      if (OrderByFieldsName <> '') then
        Add('ORDER BY ' + OrderByFieldsName);
      if (AMaxRecordsCount > 0) then
        Add('LIMIT ' + IntToStr(AMaxRecordsCount));
      Add(') as PersonsSubQuery');
      if (OrderByFieldsName = '') then
        OrderByFieldsName := 'Code'
      else
        OrderByFieldsName := OrderByFieldsName + ', Code';
      Add('ORDER BY ' + OrderByFieldsName);
    end;
  finally
    ResetRegistrySubKey();
  end;
end;



function GetPartnerImageFile(ASite: string; APicture: string): string;

begin

  Result := APicture;
  if (Result = '') then
  begin
    Result := ASite + '.gif';
    if (not MyFileExists(PicturesPath + Result)) then
      Result := '';
  end;
end;

//14.03.13
function StringReplaceIgnoreCase(AText: string; AReplaceWhat: string; AReplaceTo: string): string;
var
  TextPos: integer;
begin
  Result := AText;
  AReplaceWhat := LowerCase(AReplaceWhat);
  TextPos := Pos(AReplaceWhat, LowerCase(Result));
  while (TextPos > 0) do
  begin
    Delete(Result, TextPos, Length(AReplaceWhat));
    Insert(AReplaceTo, Result, TextPos);
    TextPos := MyPos(AReplaceWhat, LowerCase(Result), TextPos + Length(AReplaceTo));
  end;
end;

function GetFileSize(AFilePath: string): Integer;
var
  FileStream: TFileStream;
begin
  Result := 0;
  if (MyFileExists(AFilePath)) then
  begin
    FileStream := TFileStream.Create(AFilePath, fmOpenRead or fmShareDenyNone);
    try
      Result := FileStream.Size;
    finally
      FileStream.Free();
    end;
  end;
end;

// Fade. Add fix for files whose size is more than 1 Мб
function NeedToCreateThumbFile(AThumbFilePath: string): Boolean;
const
  // max file size
  MAX_FILE_SIZE = 1 * 1024 * 1024;
begin
  Result := True;
  if (MyFileExists(AThumbFilePath)) then
  begin
    Result := Result and (GetFileSize(AThumbFilePath) >= MAX_FILE_SIZE);
    if (Result) then
      DeleteFile(AThumbFilePath);
  end;
end;

function GetUserReferenceProgramType(ACode: Integer): Integer;
begin
  Result := MOVIENIZER_PROGRAM;
  if (ACode in [1000..1999]) then
    Result := MUSICNIZER_PROGRAM
  else if (ACode in [2000..2999]) then
    Result := BOOKNIZER_PROGRAM
  else if (ACode in [3000..3999]) then
    Result := GAMENIZER_PROGRAM;
end;

function GetOldReferenceCodeForNewCode(ACode: Integer): Integer;
begin
  Result := ACode;
  case GetUserReferenceProgramType(ACode) of
    MUSICNIZER_PROGRAM: Result := Result - 1000;
    BOOKNIZER_PROGRAM: Result := Result - 2000;
    GAMENIZER_PROGRAM: Result := Result - 3000;
  end;
end;


function GetShots(Code, ImageMode: Integer; MyObject: String; AdditionalImageMode: Integer=0; AdditionalImageMode2: Integer=0): String;

var
	sl: TStringList;
	i, j, kol, RecordCount: Integer;
	s, FileName, ImageModes : String;
begin
	s := '';
	if AdditionalImageMode > 0 then
		ImageModes := Format('%d, %d', [ImageMode, AdditionalImageMode])
	else
		ImageModes := IntToStr(ImageMode);
	if AdditionalImageMode2 > 0 then
		ImageModes := Format('%s, %d', [ImageModes, AdditionalImageMode2]);
	sl := TStringList.Create;
	with sl do begin
		Clear;
		Add('select path, code from images');
		Add(format('where %s=%d and mode in (%s)', [MyObject, Code, ImageModes]));
		Add('order by sort_order');
	end;
	RecordCount:=ExecuteSQL(sl.text);
	if RecordCount>0 then begin
		kol := (GetHTMLWidth - 30) div ShotWidth;
		j := 0;
		s := '<tr><td style="background:url('+WhiteBarFile+') repeat-x;">&nbsp;</td></tr><tr><td><table class="ref-boxes" align="center"><tr>';
		//skip the first photo
		if MyObject = 'person' then begin
			GetNextRecord;
			RecordCount := RecordCount - 1;
		end;
		for i:=1 to RecordCount do begin
			j := j+1;
			FileName:=ImagesPath+GetStringField('path');
			if (MyFileExists(FileName)) then
				s:=s+format('<td class="box-shots"><a class="picture" href="ShowImage:%d"><img src="%s"></a></td>', [GetIntegerField('code'), FileName]);
			if j>=kol then begin
				j := 0;
				s:=s+'</tr><tr>';
			end;
			GetNextRecord;
		end;
		s := s + '</tr></table></td></tr>';
	end;
	sl.Free;
	result := s;
end;

//shows the loan page
function GetLoans: string;
var
	all, s, loan_date, return_date: String;
	i, j, RecordCount, EditionCode, MovieCode: Integer;
	sl: TStringList;
begin
	sl := TStringList.Create;
	with sl do begin
	  Clear;
	  Add('select l.code, l.loan_date, l.due_date, l.edition, l.movie, l.comment, m.name, mv.title from loans l');
	  Add('inner join manuals m on m.code=l.location');
	  Add('inner join editions ed on ed.code=l.edition');
	  Add('inner join movies mv on mv.code=l.movie');
	  Add('where (return_date='''' or return_date is null)');
	  Add('order by mv.title_sort');
	end;
	RecordCount:=ExecuteSQL(sl.text);
	all := GetPatternTop + '<p class="title">' + GetLangStr(1440) + format(' (%d)', [RecordCount])+'</p></td>'+ClosePatternTop;
	all := all + Format('<tr><td><table class="content"><tr><td style="background:url(%s) repeat-x;">&nbsp;</td></tr>', [WhiteBarFile]);
	//no loans
	if RecordCount = 0 then
		all := all + '<tr><td>' + GetLangStr(1455) + '</td></tr>'
	else begin
		all := all + '<tr><td><table class="loans">';
		all := all + format('<tr><td class="loan-title"><b>%s</b></td><td class="loan-title"><b>%s</b></td><td class="loan-title"><b>%s</b></td><td class="loan-title"><b>%s</b></td><td class="loan-title"><b>%s</b></td><td class="loan-title"><b>%s</b></td><td class="loan-title"><b>%s</b></td></tr>', [GetLangStr(46), GetLangStr(239), GetLangStr(240), GetLangStr(241), GetLangStr(245), GetLangStr(163), GetLangStr(1446)]);
		for i:=1 to RecordCount do begin
			all := all + '<tr>';
			EditionCode := GetIntegerField('l.edition');
			MovieCode := GetIntegerField('l.movie');
			loan_date := GetSmartDate(GetStringField('l.loan_date'));
			return_date := Format('<a href="userref:11,%d;%d;%d;loan-page">%s</a>', [EditionCode, MovieCode, GetIntegerField('l.code'), GetLangStr(1445)]);
			all := all + format('<td><a href="movie:%d">%s</a></td>', [MovieCode, GetStringField('mv.title')]);
			all := all + format('<td class="loan-date">%s</td>', [loan_date]);
			all := all + format('<td class="loan-date">%s</td>', [GetSmartDate(GetStringField('l.due_date'))]);
			all := all + format('<td class="loan-date">%s</td>', [return_date]);
			all := all + format('<td>%s</td>', [GetStringField('m.name')]);
			all := all + format('<td>%s</td>', [GetStringField('l.comment')]);
			all := all + format('<td><a class="picture" href="userref:14,%d;%d;%d;loan-page"><img src="%sdelete.png"></a></td>', [EditionCode, MovieCode, GetIntegerField('l.code'), ImgPath]);
			all := all + '</tr>';
			GetNextRecord;
		end;
		all := all + '</table></td></tr>';
	end;

	all := all + '</table>' + GetPatternBottom;
	sl.Free;
	result := all;
end;

function GetCast(MovieCode: Integer): String;
var
	sl, PreviousCodes: TStringList;
	RecordCount, i, person_code, prev_code, rating, PageWidth, col, total_col: Integer;
	s, img, character, rate, BirthDate, FileName: String;
  EmptyPhotoFileName: string;
begin
	sl := TStringList.Create;
	PreviousCodes := TStringList.Create;
	s := '';
    with sl do begin
		Clear;
        Add('select p.code, p.name, p.rating, p.item_type, p.birth_date, c.name, i.path from data d');
        Add('inner join persons p on p.code=d.person');
		Add('left outer join images i on i.person=d.person');
		Add('left outer join characters c on c.person=p.code and c.movie=d.movie');
        Add(format('where d.movie=%d and d.mode=3', [MovieCode]));
        Add('order by d.sort_order, i.mode desc, i.sort_order');
    end;
    RecordCount:=ExecuteSQL(sl.text);
    if RecordCount>0 then begin
		PageWidth := GetHTMLWidth - 250;
		total_col := PageWidth div 300;
		s := s + '<tr style="background:url('+WhiteBarFile+') repeat-x;"><td>&nbsp;&nbsp;</td></tr><tr><td><table class="cast"><tr>';
        i:=1;
		prev_code := 0;
		col := 1;
        while i<=RecordCount do begin
		  person_code := GetIntegerField('p.code');
		  rating:=GetIntegerField('p.rating');
		  if rating>0 then
			rate:=format(' <img src="%sstar.gif" width=16 height=16 border=0 align="absmiddle">%d', [ImgPath, rating])
		  else
			rate:='';
		  BirthDate:=GetStringField('p.birth_date');
		  if BirthDate<>'' then
			BirthDate:=Date2Str(BirthDate);
		  if PreviousCodes.IndexOf(IntToStr(person_code)) < 0 then begin
			  PreviousCodes.Add(IntToStr(person_code));
			  //photo
			  s := s + '<td class="cast-name">';
        EmptyPhotoFileName := NO_PHOTO_FILE[GetIntegerField('p.item_type')];
			  FileName := GetImageFileName(GetStringField('i.path'), 'Thumbs', EmptyPhotoFileName, DefaultThumbWidth, 0);
			  s:=s+format('<div><div class="thumb-cover-border"><a href="person:%d"><img class="thumb-cover" src="%s"></a>', [person_code, FileName]);
			//name
			  s := s+format('</div><div class="thumb-text"><a href="person:%d"><big><b>%s</b></big></a>%s<br>', [person_code, GetStringField('p.name'), rate]);

			  //birth date
//			  if BirthDate <> '' then
//				s := s + BirthDate + '<br>';

			  //character
			  character:=GetStringField('c.name');
			  s := s + character + '</div></td>';
			  //detect how many columns to create
			  col := col + 1;
			  if col > total_col then begin
				s := s + '</tr><tr>';
				col := 1;
			  end;
			  prev_code := person_code;
		  end;
		  i:=i+1;
          GetNextRecord;
        end;
		s := s + '</tr>';
		s := s + '</table></td></tr>';
	end;
	sl.Free;
	PreviousCodes.Free;
	Result := s;
end;

//14.03.13
function GenerateHTMLForDescriptionSearching(AFullDescription: string;
  ASearchDescription: string; AMaxLength: Integer = 350): string;
var
  StartPos: Integer;
  EndPos: Integer;
  SearchPos: Integer;
begin
  Result := '';
  if ((ASearchDescription <> '') and
      (AFullDescription <> '')) then
  begin
    SearchPos := Pos(LowerCase(ASearchDescription), LowerCase(AFullDescription));
    if (SearchPos > 0) then
    begin
      // calculate start pos
      StartPos := SearchPos - ((AMaxLength - Length(ASearchDescription)) div 2);
      if (StartPos <= 0) then
        StartPos := 1
      else
      begin
        // in other case search first space from StartPos position
        if (MyPos(' ', AFullDescription, StartPos) > 0) then
          StartPos := MyPos(' ', AFullDescription, StartPos) + 1;
      end;
      // find end pos of the text
      EndPos := StartPos + AMaxLength;
      if (EndPos > Length(AFullDescription)) then
        EndPos := Length(AFullDescription)
      else
      while ((Length(AFullDescription) > 0) and
             (AFullDescription[EndPos] <> ' ')) do
      begin
        Dec(EndPos);
      end;
      //
      Result := Copy(AFullDescription, StartPos, EndPos - StartPos);
      Result := StringReplaceIgnoreCase(Result, ASearchDescription,
        '<font style="background-color: yellow">' + ASearchDescription + '</font>');
      Result := Trim(StringReplaceIgnoreCase(Result, '<br>', ' '));
      if (StartPos <> 1) then
        Result := '...' + Result;
      if (EndPos <> Length(AFullDescription)) then
        Result := Result + '...';
    end;
  end;
  if (Result <> '') then
    Result := '<div style="margin:5;">"' + Result + '"</p>';
end;

function MyExtractFileExt(s: String): String;
var
	i: Integer;
begin
	i := Pos('.', s);
	while i > 0 do begin
		delete(s, 1, i);
		i := Pos('.', s);
	end;
	result := s;
end;

function GetSortIcons(sort_order: String; PersonCode: Integer; PersonType: String; ProgramType: Integer): String;
begin
	result := '';
	if (RegReadInteger('SortIcons', 0) = 1) then begin
		if (sort_order <> 'title asc') then
			result := result + format('&nbsp;<a class="picture" href="userref:%d231,%s;%d">', [ProgramType, PersonType, PersonCode]) + format('<img alt="%s" src="%stitle-sort-ascend.png" align="absmiddle"></a>', [GetLangStr(1477), ImgPath]);
		if (sort_order <> 'title desc') then
			result := result + format('&nbsp;<a class="picture" href="userref:%d232,%s;%d">', [ProgramType, PersonType, PersonCode]) + format('<img alt="%s" src="%stitle-sort-descend.png" align="absmiddle"></a>', [GetLangStr(1478), ImgPath]);
		if (sort_order <> 'year asc') then
			result := result + format('&nbsp;<a class="picture" href="userref:%d233,%s;%d">', [ProgramType, PersonType, PersonCode]) + format('<img alt="%s" src="%syear-sort-ascend.png" align="absmiddle"></a>', [GetLangStr(1479), ImgPath]);
		if (sort_order <> 'year desc') then
			result := result + format('&nbsp;<a class="picture" href="userref:%d234,%s;%d">', [ProgramType, PersonType, PersonCode]) + format('<img alt="%s" src="%syear-sort-descend.png" align="absmiddle"></a>', [GetLangStr(1480), ImgPath]);
		if RegReadInteger('ShowRating', 1) = 1 then begin
			if (sort_order <> 'rating asc') then
				result := result + format('&nbsp;<a class="pic" href="userref:%d235,%s;%d">', [ProgramType, PersonType, PersonCode]) + format('<img alt="%s" border="0" src="%ssort-rating-asc.png" align="absmiddle"></a>', [GetLangStr(7007), ImgPath]);
			if (sort_order <> 'rating desc') then
				result := result + format('&nbsp;<a class="pic" href="userref:%d236,%s;%d">', [ProgramType, PersonType, PersonCode]) + format('<img alt="%s" border="0" src="%ssort-rating-desc.png" align="absmiddle"></a>', [GetLangStr(7008), ImgPath]);
		end;
	end;
end;

function ShowLoanManagerMicro(EditionCode, MovieCode: Integer): string;
var
	sl: TStringList;
	i, RecordCount, dd, mm, yy, loc_code: Integer;
	s, loc_name, day, month, year, due_date, loan_date: String;
	f: TForm;
	b: TButton;
	labDueDate, labComment, labLocation: TLabel;
	comboLocation: TComboBox;
	edComment, edDueDate: TEdit;

procedure ButtonClick(Sender: TButton);
begin
	s := GetLangStr(1439);
	sl := TStringList.Create;
	with sl do begin
		Clear;
		Add('SELECT l.location from loans l');
		Add('JOIN manuals m on m.code=l.location');
		Add(format('WHERE m.name=''%s''', [comboLocation.Items[comboLocation.ItemIndex]]));
	end;
	RecordCount := ExecuteSQL(sl.text);
	loc_code := GetIntegerField('l.location');
	if (edDueDate.Text <> '') then begin
		year := copy(edDueDate.Text, 1, 2);
		month := copy(edDueDate.Text, 4, 2);
		day := copy(edDueDate.Text, 7, 2);
		try
			dd := StrToInt(day);
			mm := StrToInt(month);
			yy := StrToInt(year);
		except
			ShowMessage(GetLangStr(1444));
		end;
	end;
	if ((dd>0) and (mm>0) and (yy>0)) or (edDueDate.Text = '') then begin
		due_date := format('%d/%d/20%d', [mm, dd, yy]);
		DecodeDate(Date, yy, mm, dd);
		loan_date := format('%d/%d/%d', [mm, dd, yy]);
		if edDueDate.Text = '' then
			due_date := '';
		with sl do begin
			Clear;
			Add('INSERT INTO loans');
			Add('(movie, edition, loan_date, due_date, comment, location)');
			Add(Format('VALUES (%d, %d, ''%s'', ''%s'', ''%s'', %d)', [MovieCode, EditionCode, loan_date, due_date, edComment.text, loc_code]));
		end;
		RecordCount := ExecuteSQL(sl.text);
		f.ModalResult := mrOk;
	end;
	sl.Free;
end;

begin
	s:=GetLangStr(1335);
	f := TForm.Create(nil);
	f.Caption := GetLangStr(1439);
	f.BorderStyle := bsDialog;
	f.Position := poScreenCenter;
	f.Width:=180;
	f.Height:=200;
	f.AutoSize:=false;

	labLocation := TLabel.Create(f);
	labLocation.Name := 'labLocation';
	labLocation.Parent := f;
	labLocation.SetBounds(10, 10, 150, 20);
	labLocation.Caption := GetLangStr(178);

	comboLocation := TComboBox.Create(f);
	comboLocation.Name := 'comboLocation';
	comboLocation.Parent := f;
	comboLocation.SetBounds(10, 25, 150, 20);
	comboLocation.Style := csDropDownList;
	sl := TStringList.Create;
	with sl do begin
		Clear;
		Add('select distinct name from manuals');
		Add('where reference=10');
	end;
	RecordCount := ExecuteSQL(sl.text);
	for i:=1 to RecordCount do begin
		comboLocation.Items.Add(GetStringField('name'));
		GetNextRecord;
	end;

    with sl do begin
        Clear;
        Add('select m.name from loans l');
        Add('inner join manuals m on m.code=l.location');
        Add(format('where edition=%d', [EditionCode]));
	end;
	RecordCount := ExecuteSQL(sl.text);
	loc_name := '';
	if RecordCount > 0 then
		loc_name := GetStringField('m.name');
	if (loc_name <> '') then begin
		comboLocation.ItemIndex := comboLocation.Items.IndexOf(loc_name);
	end;

	sl.Free;

	labComment := TLabel.Create(f);
	labComment.Name := 'labComment';
	labComment.Parent := f;
	labComment.SetBounds(10, 55, 150, 20);
	labComment.Caption := GetLangStr(163);

	edComment := TEdit.Create(f);
	edComment.Name := 'edComment';
	edComment.Parent := f;
	edComment.SetBounds(10, 70, 150, 20);
	edComment.Text := '';

	labDueDate := TLabel.Create(f);
	labDueDate.Name := 'labDueDate';
	labDueDate.Parent := f;
	labDueDate.SetBounds(10, 95, 150, 20);
	labDueDate.Caption := GetLangStr(240) + format(' (%s)', [GetLangStr(1443)]);

	edDueDate := TEdit.Create(f);
	edDueDate.Name := 'edDueDate';
	edDueDate.Parent := f;
	edDueDate.SetBounds(10, 110, 150, 20);
	edDueDate.Text := '';

	b := TButton.Create(f);
	b.Name := 'btnOK';
	b.Parent := f;
	b.SetBounds(50, 140, 80, 25);
	b.Caption := '&OK';
	b.OnClick := @ButtonClick;

	f.ShowModal;
	f.Free;

	result:=s;

end;



(*counts how many movies/persons would fit the previous pages (before the current page),
or how many pages would be needed for the given number of movies/persons
(after the current page)
returns the number of movies that would fit the given number of pages
saves the number of processed records to the ProcessedRecords global variable
saves the number of counted pages to the CountedPages global variable *)
function CountItemsForPages(page_width, prev_page, RecordCount, i, lines, ItemHeight, StandardWidth: Integer): Integer;
var
	page_width_copy, LastCode, my_code, j, PicWidth, TotalItems, total_pages, PicHeight, lines_copy: Integer;
	PreviousCodes: TStringList;
begin
  page_width_copy := page_width;
	LastCode := 0;
	lines_copy := lines;
	j := 0;
	TotalItems := 0;
	total_pages := 0;
	PreviousCodes := TStringList.Create;
	while (i<RecordCount) and (prev_page > 1) do begin
		my_code := GetIntegerField('code');
		//disregard duplicate items
		if (PreviousCodes.IndexOf(IntToStr(my_code)) < 0) then begin
			PreviousCodes.Add(IntToStr(my_code));
			//check the width of every item
			PicHeight := GetIntegerField('i.height');
			if PicHeight > 0 then
				PicWidth := Round(GetIntegerField('i.width') * (ItemHeight / PicHeight)) + NormalImageBorders
			else
				PicWidth := StandardWidth;
			page_width_copy := page_width_copy - PicWidth;
			LastCode := my_code;
			//if it still fits the page, count it
			if (page_width_copy >= 0) then begin
				j := j + 1;
			end
			//if the item doesn't fit the page anymore, remember how many items fit the page
			else if lines_copy > 1 then begin
				//proceed to the next line
				page_width_copy := page_width - PicWidth;
				TotalItems := TotalItems + j;
				//the current record goes to the next line
				j := 1;
				lines_copy := lines_copy - 1;
			end
			//next page
			else begin
				lines_copy := lines;
				TotalItems := TotalItems + j;
				total_pages := total_pages + 1;
				prev_page := prev_page - 1;
				page_width_copy := page_width - PicWidth;
				//if prev_page <= 1, the loop will be broken, and the record
				//will go to the current page (j := 0), otherwise the record will
				//go to another previous page (j := 1)
				if (prev_page > 1) then
					j := 1
				else
					j := 0;
			end;
		end
		else begin
			GetNextRecord;
			i := i + 1;
		end;
	end;
	if j > 0 then begin
		total_pages := total_pages + 1;
		TotalItems := TotalItems + j;
	end;
	PreviousCodes.Free;
	//a global variable is used to return the second result
	ProcessedRecords := i;
	//a global variable is used to return the third result
	CountedPages := total_pages;
	Result := TotalItems;
end;

function ProcessMovieReference(AProgramType: Integer;
  text: String; page: integer; header_img: String = ''): String;
var
  i, j, kol, RecordCount, rating, page_width, LastCode, my_i, my_code: integer;
  s, description, year, director, writer, actor, bio, character, movie, movie_top: string;
  all, FileName, img, rate, BackgroundMovieRef: string;
  IMDBRating: double;
  ii, TotalItems, PageItems, lines, PicWidth, PicHeight, page_width_copy: integer;
  fl: boolean;
  PreviousCodes: TStringList;
begin
	all := '';
  SetRegistrySubKey(REGISTRY_SUB_KEYS[AProgramType]);
  try
    BackgroundMovieRef := ImgPath + 'background-movies-ref.gif';
    BackgroundMovieRef := ReplaceStr(BackgroundMovieRef, '(', '%28');
    BackgroundMovieRef := ReplaceStr(BackgroundMovieRef, ')', '%29');
    BackgroundMovieRef := ReplaceStr(BackgroundMovieRef, ' ', '%20');
    PreviousCodes := TStringList.Create;

    CountedPages := 0;
    page_width:=GetHTMLWidth - WholePageBorder;

    GetFirstRecord();
    RecordCount := GetRecordsCount();


    lines := RegReadInteger('LinesMovies', DefaultLinesMovies);
    //count how many items were listed on the previous pages
    TotalItems := CountItemsForPages(page_width, page, RecordCount, 0, lines, CoverHeight, StandardCoverWidth[AProgramType]);
    i := ProcessedRecords;

    LastCode:=0;
    fl:=true;
    PageItems := 0;
    page_width_copy := page_width;
    movie := '';
    //list movies on the current page
    while (i<RecordCount) and fl do begin
      my_code:=GetIntegerField('code');
      if (PreviousCodes.IndexOf(IntToStr(my_code)) < 0) then begin
        PreviousCodes.Add(IntToStr(my_code));
        LastCode:=my_code;
        rating:=GetIntegerField('rating');
        rate:='';
        PicHeight := GetIntegerField('height');
        if PicHeight > 0 then
          PicWidth := Round(GetIntegerField('width') * (CoverHeight / PicHeight)) + NormalImageBorders
        else
          PicWidth := StandardCoverWidth[AProgramType];
        page_width_copy := page_width_copy - PicWidth;
        if (page_width_copy < 0) then begin
          all := all + movie + '</tr></table></tr></td><tr><td align="center"><table class="ref-boxes"><tr>';
          movie := '';
          lines := lines - 1;
          page_width_copy := page_width - PicWidth;
        end;
        if (page<>0) and (lines <= 0) then
          fl:=false
        else begin
          if rating > 0 then begin
            for my_i:=1 to 10 do begin
              if my_i mod 2=1 then begin
                if my_i<=rating then
              rate:=rate+format('<img src="%sstar-left.gif" align="absmiddle" border="0">',  [ImgPath])
                else
              rate:=rate+format('<img src="%snot-star-left.gif" align="absmiddle" border="0">',  [ImgPath])
              end
              else begin
                if my_i<=rating then
              rate:=rate+format('<img src="%sstar-right.gif" align="absmiddle" border="0">',  [ImgPath])
                else
              rate:=rate+format('<img src="%snot-star-right.gif" align="absmiddle" border="0">',  [ImgPath])
              end;
            end;
          end;
          IMDBRating := GetFloatField('imdb_rating');

          FileName := GetImageFileName(GetStringField('path'), 'ThumbsHomeMovies', NO_COVER_FILE[AProgramType], 0, CoverHeight);
          movie:=movie+'<td class="box-movie-ref" style="background:url('+BackgroundMovieRef+') repeat-x;">'+format('<a href="movie:%d"><img class="movie-cover" src="', [my_code])+FileName+'"></a>';

          if (rate<>'') and (RegReadInteger('ShowRating', 1) = 1) then
            movie:=movie+'<br>'+rate;
          movie := movie + '<p>';
          if IMDBRating <> 0 then
            movie:=movie+format('IMDB: %0.1f<br>', [IMDBRating]);
          //sequence number
          if TempReference = 22 then
            movie:=movie+GetLangStr(529)+IntToStr(GetIntegerField('series_nom')) + ': ';
          movie := movie+format('<a href="movie:%d">%s</a></p></td>', [my_code, GetStringField('title')]);

          PageItems:=PageItems+1;
          GetNextRecord;
          i := i+1;
        end;
      end
      else begin
        GetNextRecord;
        i:=i+1;
      end;
    end;//while

    TotalItems := TotalItems + PageItems;


    all:=all+movie+'</tr></table></td></tr>';
    all := all + '</table>';
    kol := 0;
    lines := RegReadInteger('LinesMovies', DefaultLinesMovies);
    if (page > 0) and (i < RecordCount) then begin
      //set the maximum number of pages to 2000
      TotalItems := TotalItems + CountItemsForPages(page_width, 2000, RecordCount, i, lines, CoverHeight, StandardCoverWidth[AProgramType]);
      kol := CountedPages;
    end;

    //get the top of the page (we need to know how many movies were there)
    movie_top:=GetPatternTop;
    movie_top:=movie_top + '<p class="title">' + text + format(' (%d)', [TotalItems]);
    movie_top:=movie_top+'</p></td>'+ClosePatternTop;
    movie_top := movie_top + Format('<tr><td><table class="content"><tr><td style="background:url(%s) repeat-x;">&nbsp;</td></tr>', [WhiteBarFile]);
    movie_top:=movie_top+'<tr><td align="center"><table class="ref-boxes"><tr>';

    case TempReference of
      1: s:='video_standard:'+IntToStr(TempReferenceCode);
      2: s:='media_format:'+IntToStr(TempReferenceCode);
      3: s:='media_type:'+IntToStr(TempReferenceCode);
      4: s:='aspect_ratio:'+IntToStr(TempReferenceCode);
      5: s:='resolution:'+IntToStr(TempReferenceCode);
      6: s:='subtitle:'+IntToStr(TempReferenceCode);
      7: s:='audio_track:'+IntToStr(TempReferenceCode);
      8: s:='genre:'+IntToStr(TempReferenceCode);
      9: s:='country:'+IntToStr(TempReferenceCode);
      10: s:='location:'+IntToStr(TempReferenceCode);
      11: s:='mpaa:'+IntToStr(TempReferenceCode);
      12: s:='studio:'+IntToStr(TempReferenceCode);
      13: s:='custom1:'+IntToStr(TempReferenceCode);
      14: s:='custom2:'+IntToStr(TempReferenceCode);
      15: s:='custom3:'+IntToStr(TempReferenceCode);
      16: s:='custom4:'+IntToStr(TempReferenceCode);
      17: s:='custom5:'+IntToStr(TempReferenceCode);
      18: s:='custom6:'+IntToStr(TempReferenceCode);
      19: s:='video_codec:'+IntToStr(TempReferenceCode);
      20: s:='disc_label:'+IntToStr(TempReferenceCode);
      22: s:='sequence:'+IntToStr(TempReferenceCode);
      23: s:='book_language:'+IntToStr(TempReferenceCode);
      300: s:='year:'+IntToStr(TempReferenceCode) + ',' + IntToStr(AProgramType);
      301: s := 'show_my_rating:' + IntToStr(TempReferenceCode) + ',' + IntToStr(AProgramType);
      666: s:='userref:'+IntToStr(TempReferenceCode)+','+TempText;
    else
      s:='';
    end;
    if (page > 1) or (kol > 0) then begin
      all:=all+'<center><table class="navi"><td class="navi-border">'+format('<img src="%snavi-left.gif"></td><td style="background:url(%s) repeat-x;"><p>', [ImgPath, WhiteBarFile]);
      if (page>1) or (kol > 0) then
        all:=all+format('<a href="%s,%d">%s</a>   ', [s, 0, GetLangStr(1330)]);
      if page>1 then
        all:=all+format('<a href="%s,%d">%s</a>   ', [s, page-1, GetLangStr(1328)]);
      for i:=1 to page do begin
        if page=i then
          all:=all+format(' <b>%d</b> ', [i])
        else
          all:=all+format(' <a href="%s,%d">%d</a> ', [s, i, i]);
      end;
      for i:=(page+1) to (page+kol) do
        all:=all+format(' <a href="%s,%d">%d</a> ', [s, i, i]);
      if kol>0 then
        all:=all+format('   <a href="%s,%d">%s</a> ', [s, page+1, GetLangStr(1329)]);
      all:=all+format('</p></td><td class="navi-border"><img src="%snavi-right.gif"></td></table></center>', [ImgPath]);
    end;


    all:=movie_top+all+GetPatternBottom;
    PreviousCodes.Free;
    Result := all;

  finally
    ResetRegistrySubKey();
  end;
end;

function ProcessPersonReference(AProgramType: Integer;
  text: String; page: integer; header_img: String = ''): String;
var
  i, j, kol, RecordCount, rating, page_width, LastCode, my_i, my_code: integer;
  s, description, year, director, writer, actor, bio, character, person, person_top: string;
  all, FileName, img, rate, BackgroundMovieRef: string;
  IMDBRating: double;
  ii, TotalItems, PageItems, lines, PicWidth, PicHeight, page_width_copy: integer;
  fl: boolean;
  PreviousCodes: TStringList;
begin
	all := '';
  SetRegistrySubKey(REGISTRY_SUB_KEYS[AProgramType]);
  try
    PreviousCodes := TStringList.Create;

    CountedPages := 0;
    page_width:=GetHTMLWidth - WholePageBorder;

    RecordCount := GetRecordsCount();
    lines := RegReadInteger('LinesPeople', DefaultLinesPeople);
    //count how many items were listed on the previous pages
    TotalItems := CountItemsForPages(page_width, page, RecordCount, 0, lines, PhotoHeight, StandardPhotoWidth);
    i := ProcessedRecords;

    LastCode:=0;
    fl:=true;
    PageItems := 0;
    page_width_copy := page_width;
    person := '';
    //list persons on the current page
    while (i<RecordCount) and fl do begin
      my_code:=GetIntegerField('code');
      if (PreviousCodes.IndexOf(IntToStr(my_code)) < 0) then begin
        PreviousCodes.Add(IntToStr(my_code));
        LastCode:=my_code;
        rating:=GetIntegerField('rating');
        rate:='';
        PicHeight := GetIntegerField('height');
        if PicHeight > 0 then
          PicWidth := Round(GetIntegerField('width') * (PhotoHeight / PicHeight)) + NormalImageBorders
        else
          PicWidth := StandardPhotoWidth;
        page_width_copy := page_width_copy - PicWidth;
        if (page_width_copy < 0) then begin
          all := all + person + '</tr></table></tr></td><tr><td align="center"><table class="ref-boxes"><tr>';
          person := '';
          lines := lines - 1;
          page_width_copy := page_width - PicWidth;
        end;
        if (page<>0) and (lines <= 0) then
          fl:=false
        else begin
          if rating > 0 then begin
            for my_i:=1 to 10 do begin
              if my_i mod 2=1 then begin
                if my_i<=rating then
              rate:=rate+format('<img src="%sstar-left.gif" align="absmiddle" border="0">',  [ImgPath])
                else
              rate:=rate+format('<img src="%snot-star-left.gif" align="absmiddle" border="0">',  [ImgPath])
              end
              else begin
                if my_i<=rating then
              rate:=rate+format('<img src="%sstar-right.gif" align="absmiddle" border="0">',  [ImgPath])
                else
              rate:=rate+format('<img src="%snot-star-right.gif" align="absmiddle" border="0">',  [ImgPath])
              end;
            end;
          end;
          IMDBRating := GetFloatField('imdb_rating');

          FileName := GetImageFileName(GetStringField('path'), 'ThumbsPhotos', NO_PHOTO_FILE[AProgramType], 0, PhotoHeight);
          person:=person+'<td class="box-photos" style="background:url('+BackgroundPeople+') repeat-x;">'+format('<a href="person:%d"><img class="person-photo" src="', [my_code])+FileName+'"></a>';

          if (rate<>'') and (RegReadInteger('ShowRating', 1) = 1) then
            person:=person+'<br>'+rate;
          person := person + '<p>';
          person := person+format('<a href="person:%d">%s</a></p></td>', [my_code, GetStringField('name')]);

          PageItems:=PageItems+1;
          GetNextRecord;
          i := i+1;
        end;
      end
      else begin
        GetNextRecord;
        i:=i+1;
      end;
    end;//while

    TotalItems := TotalItems + PageItems;


    all:=all+person+'</tr></table></td></tr>';
    all := all + '</table>';
    kol := 0;
    lines := RegReadInteger('LinesPeople', DefaultLinesPeople);
    if (page > 0) and (i < RecordCount) then begin
      //set the maximum number of pages to 2000
      TotalItems := TotalItems + CountItemsForPages(page_width, 2000, RecordCount, i, lines, PhotoHeight, StandardPhotoWidth);
      kol := CountedPages;
    end;

    //get the top of the page (we need to know how many persons were there)
    person_top:=GetPatternTop;
    person_top:=person_top + '<p class="title">' + text + format(' (%d)', [TotalItems]);
    person_top:=person_top+'</p></td>'+ClosePatternTop;
    person_top := person_top + Format('<tr><td><table class="content"><tr><td style="background:url(%s) repeat-x;">&nbsp;</td></tr>', [WhiteBarFile]);
    person_top:=person_top+'<tr><td align="center"><table class="ref-boxes"><tr>';

    case TempReference of
      24: s:='custom_person1:'+IntToStr(TempReferenceCode);
      25: s:='custom_person2:'+IntToStr(TempReferenceCode);
      26: s:='custom_person3:'+IntToStr(TempReferenceCode);
      27: s:='custom_person4:'+IntToStr(TempReferenceCode);
      28: s:='custom_person5:'+IntToStr(TempReferenceCode);
      29: s:='custom_person6:'+IntToStr(TempReferenceCode);
    else
      s:='';
    end;
    if (page > 1) or (kol > 0) then begin
      all:=all+'<center><table class="navi"><td class="navi-border">'+format('<img src="%snavi-left.gif"></td><td style="background:url(%s) repeat-x;"><p>', [ImgPath, WhiteBarFile]);
      if (page>1) or (kol > 0) then
        all:=all+format('<a href="%s,%d">%s</a>   ', [s, 0, GetLangStr(1330)]);
      if page>1 then
        all:=all+format('<a href="%s,%d">%s</a>   ', [s, page-1, GetLangStr(1328)]);
      for i:=1 to page do begin
        if page=i then
          all:=all+format(' <b>%d</b> ', [i])
        else
          all:=all+format(' <a href="%s,%d">%d</a> ', [s, i, i]);
      end;
      for i:=(page+1) to (page+kol) do
        all:=all+format(' <a href="%s,%d">%d</a> ', [s, i, i]);
      if kol>0 then
        all:=all+format('   <a href="%s,%d">%s</a> ', [s, page+1, GetLangStr(1329)]);
      all:=all+format('</p></td><td class="navi-border"><img src="%snavi-right.gif"></td></table></center>', [ImgPath]);
    end;


    all:=person_top+all+GetPatternBottom;
    PreviousCodes.Free;
    Result := all;
  finally
    ResetRegistrySubKey();
  end;
end;




begin

end.
